from __future__ import absolute_import, division, print_function
from __future__ import unicode_literals

import datetime
import hashlib
import os
import random
import re
import string


def today():
    return get_date_string(datetime.date.today())


def tomorrow():
    return get_date_string(datetime.date.today() + datetime.timedelta(days=1))


def get_date_string(date_obj):
    return date_obj.strftime("%Y_%m_%d")


def is_valid_date(date_string):
    if not date_string:
        return False
    try:
        datetime.datetime.strptime(date_string, '%Y_%m_%d')
    except ValueError:
        print("Incorrect data format, should be yyyy_mm_dd")
        exit(1)
    return True


def ensure_directory(dir_name):
    if not os.path.exists(dir_name):
        os.makedirs(dir_name)


def write_file(directory, content):
    with open(f'{directory}/{get_hash(content)}', 'w') as f:
        f.write(content)


def read_file(directory, file_name):
    with open(f'{directory}/{file_name}', 'r') as f:
        content = f.read()
    return content


def move_file(directory_src, directory_dst, file_name):
    src = f'{directory_src}/{file_name}'
    dst = f'{directory_dst}/{file_name}'
    os.rename(src, dst)


def remove_file(directory, file_name):
    os.remove(f'{directory}/{file_name}')


def random_string(string_length=32):
    """Generate a random string of fixed length """
    letters = string.ascii_lowercase
    return ''.join(random.choice(letters) for i in range(string_length))


def get_hash(content):
    sha = hashlib.sha256()
    sha.update(content.encode())
    sha.update(random_string().encode())
    return sha.hexdigest()[:32]


def loop_dir(directory):
    return [f for f in os.listdir(directory) if not f.startswith('.')]



#!/usr/bin/env python
from __future__ import absolute_import, division, print_function
from __future__ import unicode_literals


import argparse
import pkg_resources
import sys

import bullet_journal
from bullet_journal import exceptions, utils


def _registered_commands(group='bullet_journal.registered_commands'):
    registered_commands = pkg_resources.iter_entry_points(group=group)
    return {c.name: c for c in registered_commands}


def dispatch(argv):
    registered_commands = _registered_commands()
    parser = argparse.ArgumentParser(prog="bj")
    parser.add_argument(
        "--version",
        action="version",
        version="Bullet Journal Version {}".format(
            bullet_journal.__version__,
        ),
    )
    parser.add_argument(
        "command",
        choices=registered_commands.keys(),
    )
    parser.add_argument(
        "args",
        help=argparse.SUPPRESS,
        nargs=argparse.REMAINDER,
    )

    args = parser.parse_args(argv)

    main = registered_commands[args.command].load()

    return main(args.args)


def main():
    utils.ensure_directory('resources')
    if len(sys.argv) == 1:
        return f'Welcome to Bullet Journal Toolkit Version {bullet_journal.__version__}!'
    try:
        return dispatch(sys.argv[1:])
    except exceptions.BulletJournalException as exc:
        return '{}: {}'.format(exc.__class__.__name__, exc.args[0])


if __name__ == "__main__":
    sys.exit(main())



from __future__ import absolute_import, division, print_function
from __future__ import unicode_literals

import argparse

from bullet_journal import utils


def daily_log(date):
    utils.is_valid_date(date)
    directory_add = f'resources/{date}/add'
    directory_close = f'resources/{date}/close'
    directory_migrate = f'resources/{date}/migrate'
    utils.ensure_directory(directory_add)
    utils.ensure_directory(directory_close)
    utils.ensure_directory(directory_migrate)
    for f in utils.loop_dir(directory_add):
        print(f"+ {utils.read_file(directory_add, f)}")
    for f in utils.loop_dir(directory_close):
        print(f"x {utils.read_file(directory_close, f)}")
    for f in utils.loop_dir(directory_migrate):
        print(f"> {utils.read_file(directory_migrate, f)}")


def main(args):
    parser = argparse.ArgumentParser(prog="bj daily_log")
    parser.add_argument(
        "date",
        nargs="?",
        default=utils.today(),
        metavar="date",
        help="The date to query",
    )

    args = parser.parse_args(args)

    # Call the check function with the arguments from the command line
    return daily_log(args.date)



from __future__ import absolute_import, division, print_function
from __future__ import unicode_literals

import argparse

from bullet_journal import utils


def add(date):
    utils.is_valid_date(date)
    directory = f'resources/{date}/add'
    utils.ensure_directory(directory)
    content = input(f"Add item to {date}: ")
    utils.write_file(directory, content)


def main(args):
    parser = argparse.ArgumentParser(prog="bj add")
    parser.add_argument(
        "date",
        nargs="?",
        default=utils.today(),
        metavar="date",
        help="The date to add item to",
    )

    args = parser.parse_args(args)

    # Call the check function with the arguments from the command line
    return add(args.date)



from __future__ import absolute_import, division, print_function
from __future__ import unicode_literals

import argparse

from bullet_journal import utils


def migrate(date):
    utils.is_valid_date(date)
    directory_add = f'resources/{date}/add'
    directory_migrate = f'resources/{date}/migrate'
    utils.ensure_directory(directory_add)
    utils.ensure_directory(directory_migrate)
    records = list()
    print(f"Active items in {date}:")
    for f in utils.loop_dir(directory_add):
        records.append(f)
        print(f"Item {len(records)}: {utils.read_file(directory_add, f)}")
    if not records:
        print(f"There is no active item in {date}.")
        exit(0)
    choices = f"1-{len(records)}" if len(records) > 1 else "1"
    index = input(f"Index of item to migrate in {date} [{choices}]: ")
    try:
        i = int(index)
    except ValueError:
        #Handle the exception
        print("Please enter an integer")
        exit(1)
    file_name = records[i-1]
    utils.move_file(directory_add, directory_migrate, file_name)

    dst_date = input(f"Date to move item to [yyyy_mm_dd] (default {utils.tomorrow()}):")
    if not utils.is_valid_date(dst_date):
        dst_date = utils.tomorrow()
        print(f"Migrated item to default date: {dst_date}")
    directory_dst = f'resources/{dst_date}/add'
    utils.ensure_directory(directory_dst)
    utils.write_file(directory_dst, utils.read_file(directory_migrate, file_name))


def main(args):
    parser = argparse.ArgumentParser(prog="bj migrate")
    parser.add_argument(
        "date",
        nargs="?",
        default=utils.today(),
        metavar="date",
        help="The date to migrate item from",
    )

    args = parser.parse_args(args)

    # Call the check function with the arguments from the command line
    return migrate(args.date)



from distutils.core import setup

import bullet_journal

setup(
    name = bullet_journal.__title__,
    packages = ['bullet_journal', 'bullet_journal.commands'],
    version = bullet_journal.__version__,
    license='MIT',
    description = 'Toolkit to make bullet journaling simple and easy',
    author = 'Pinxiu Gong',
    author_email = 'pinxiu.gong@gmail.com',
    url = 'https://github.com/pinxiu/bullet_journal',
    download_url = 'https://github.com/pinxiu/bullet_journal/archive/v_06.tar.gz',
    keywords = ['bullet', 'journal', 'toolkit', 'command line'],
    install_requires=[
        'validators',
        'beautifulsoup4',
    ],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],
    entry_points={
        'bullet_journal.registered_commands': [
            'add = bullet_journal.commands.add:main',
            'close = bullet_journal.commands.close:main',
            'delete = bullet_journal.commands.delete:main',
            'migrate = bullet_journal.commands.migrate:main',
            'daily_log = bullet_journal.commands.daily_log:main',
        ],
        'console_scripts': [
            'bj = bullet_journal.__main__:main',
        ],
    },
)
